﻿// Even-Odd Demo
// InputBox.cs
// Form that will be displayed as dialog box
// Performs validation on controls using
// Validating/Validated events.
// Provides InputValue property to get/set
// value in textbox on dialog box form.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace EvenOdd
{
    public partial class InputBox : Form
    {
        // Precondition:  None
        // Postcondition: The InputBox GUI is initialized
        public InputBox()
        {
            InitializeComponent();
        }

        internal string InputValue // Can be accessed by other classes in same namespace
        {
            // Precondition:  None
            // Postcondition: Text in inputTxt is returned
            get { return inputTxt.Text; }

            // Precondition:  None
            // Postcondition: Text in inputTxt is set to specified value
            set { inputTxt.Text = value; }
        }

        // Precondition:  Attempting to change focus from inputTxt
        // Postcondition: If entered value is valid int, focus will change,
        //                else focus will remain and error provider message set
        private void inputTxt_Validating(object sender, CancelEventArgs e)
        {
            int number; // Value entered into inputTxt

            // Will try to parse text as int
            // If fails, TryParse returns false
            // If succeeds, TryParse returns true and number stores parsed value
            if (!int.TryParse(inputTxt.Text, out number))
            {
                e.Cancel = true; // Stops focus changing process
                                 // Will NOT proceed to Validated event

                errorProvider1.SetError(inputTxt, "Enter an integer!"); // Set error message

                inputTxt.SelectAll(); // Select all text in inputTxt to ease correction
            }
            else
            {
                if (number < 0)
                {
                    e.Cancel = true; // Stops focus changing process
                    // Will NOT proceed to Validated event

                    errorProvider1.SetError(inputTxt, "Enter a non-negative integer!"); // Set error message

                    inputTxt.SelectAll(); // Select all text in inputTxt to ease correction
                }
            }
        }

        // Precondition:  inputTxt_Validating succeeded
        // Postcondition: Any error message set for inputTxt is cleared
        //                Focus is allowed to change
        private void inputTxt_Validated(object sender, EventArgs e)
        {
            errorProvider1.SetError(inputTxt, ""); // Clears error message
        }

        // Precondition:  User has initiated click on cancelBtn
        // Postcondition: If left-click, InputBox is dismissed with Cancel result
        private void cancelBtn_MouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left) // Was it a left-click?
                this.DialogResult = DialogResult.Cancel;
        }

        // Precondition:  User has initiated click on OkBtn
        // Postcondition: If all controls on form validate, InputBox is dismissed with OK result
        private void OkBtn_Click(object sender, EventArgs e)
        {
            if (this.ValidateChildren())
                this.DialogResult = DialogResult.OK;
        }

        private void InputBox_Load(object sender, EventArgs e)
        {

        }
    }
}
